#!/bin/bash

correct_clock_skew () {
	basename=$1; shift

	tfile=$basename.transit.gz
	tfilelen=`cat $basename.loss_reordering | awk '/num_recv/ {print $3}'`

	# if no tfile or len, exit
	if [[ ! -s $tfile || -z $tfilelen ]]
	then
		return 1
	fi

	gunzip -c $tfile | ./skew_lp $tfilelen > $basename.skew_data

	# if skew_lp failed
	if [ $? -ne 0 ] 
	then
		return 1
	fi

	alpha=`cat $basename.skew_data | awk '$1 !~ /^#.*/ {print $3}'`
	beta=`cat $basename.skew_data | awk '$1 !~ /^#.*/ {print $7}'`

	# Note: skew_lp returns (alpha-1) and (-beta),
	#       so we use ALPHA and -BETA in correction

	# we're also producing a files for corrected_transit before/after losses
	# matching uses timestamp from the TAL file and the one in the transit file
	# CTBL uses the previous corrected transit value- this should work...

	gunzip -c $tfile | awk -v ALPHA=$alpha -v BETA=$beta -v BASENAME=$basename '
		BEGIN {
			tal_file  = BASENAME ".transit_after_loss"
			ctal_file = BASENAME ".corrected_transit_after_loss"
			ctbl_file = BASENAME ".corrected_transit_before_loss"

			getline < tal_file;
			tal_event = $1;
			tal_time = $2;
		}
		{			
			corrected = ($2 - ((ALPHA)*($1-$2)) - BETA)
			print $1, corrected

			if ($1 == tal_time) {
				print tal_event, tal_time, corrected > ctal_file;
				print tal_event, tal_time, prev_corrected > ctbl_file;
				getline < tal_file;
				tal_event = $1;
				tal_time = $2;		
			}
			
			prev_corrected = corrected;
		}
	' > $basename.corrected_transit
	
	gunzip -c $basename.bitmap.gz | awk -v BASENAME=$basename '
		BEGIN {
			ct_file = BASENAME ".corrected_transit"

			getline < ct_file;
			ct_atime = $1;
			ct_transit = $2;
		}
		{
			if ($1 == 1) {
				print $1, $2
			} else {
				if ($3 == ct_atime) {
					print $1, $2, $3, ct_transit
					getline < ct_file;
					ct_atime = $1;
					ct_transit = $2;				
				}
			}
		}
	' > $basename.corrected_bitmap
	
	if [ -e $basename.corrected_transit ]
	then 
		rm -f $basename.corrected_transit.gz && gzip -9 $basename.corrected_transit 
	fi
	if [ -e $basename.corrected_bitmap ]
	then 
		rm -f $basename.corrected_bitmap.gz && gzip -9 $basename.corrected_bitmap 
	fi
}

